;---------------------------------------------------------
; muestreo.asm
; Programa para sacar por el DAC lo que entra por el ADC
; Guardando tambien en memoria n valores para analizarlos
;---------------------------------------------------------
          .start  "AICTEST",0x809802 ;aqui cza a ensamblar
          .sect   "AICTEST"          ;
;-----------------------------------------
; Define constantes  usadas en el programa
;-----------------------------------------
; Direcciones de los registros de control del timer 0.
; El Timer 0 es usado para generar el Master Clock.
; El master clock es usado por el AIC para generar
; las frecuencias de  conversion del ADC del DAC
; y para los filtros de capacidades conmutadas
;
T0_ctrl   .set   0x808020            ; TIM0 gl control
T0_count  .set   0x808024            ; TIM0 count
T0_prd    .set   0x808028            ; TIM0 prd
;
; Direcciones de los registros de la puerta serie
;
S0_gctrl  .set   0x808040            ; SP 0 global control
S0_xctrl  .set   0x808042            ; SP 0 FSX/DX/CLKX port ctl
S0_rctrl  .set   0x808043            ; SP 0 FSR/DR/CLKR port ctl
S0_xdata  .set   0x808048            ; SP 0 Data transmit
S0_rdata  .set   0x80804C            ; SP 0 Data receive
;
; Valores de los registros del AIC que definen las
; frecuencias de conversion del DAC (TB) y del  ADC(RB)
; frec. de conversion = frec. del SCF / contador B
; frec. del SCF = Master Clock / (2 * Contador A)
; Master Clock =  Timer cero del DSP  
;
TA        .set   11                  ; 
TB        .set   15                  ;
RA        .set   11                  ;
RB        .set   15
;
; GIE es una constante que se usa para poner en uno el bit GIE
; del registro ST que habilita las interrupciones
;
GIE       .set   0x2000              ;
;--------------------------------------
; Define algunas constantes  como datos 
;--------------------------------------
; Si la primer palabra que se envia al AIC tiene los dos bits menos 
; significativos en uno indica que la proxima es una palabra secundaria.
; Luego, de acuerdo a los dos bits menos significativos de la palabra
; secundaria se realiza una tarea diferente
; (cf manual USER'S GUIDE STARTER KIT  pag 4-17).
; 
; Se acomodan los valores de los registros TA ,RA, TB y RB
; de acuerdo al formato de la palabra secundaria (cf. manual citado)
;
A_REG        .word  (TA<<9)+(RA<<2)+0 ; A registers
B_REG        .word  (TB<<9)+(RB<<2)+2 ; B registers
C_REG        .word  00000111b         ; control
ultimo       .word  TABLA             ; direcc. de ultimo dato de tabla
veces        .word   100
;
; Valores que se cargaran en los registros de control de la puerta serie 0
;
S0_gctrl_val .word  0x0E970300        ; 
S0_xctrl_val .word  0x00000111        ;
S0_rctrl_val .word  0x00000111        ;
;

ADC_last     .word  0                 ; Last received ADC value
;********************************
;El programa principal es un lazo
;********************************
main      or    GIE,ST          ; Turn on INTS
          ldi   0xF4,IE         ; Enable XINT/RINT/INT2
          b     main            ; Do it again!
;-------------------------------
DAC2      push  ST              ; DAC Interrupt service routine
          push  R3              
          ldi   @ADC_last,R3    
          lsh     2,R3
          cmpi   32767,R3
          ldigt  32767,R3
          cmpi  -32768,R3
          ldilt -32768,R3
          andn  3,R3            
          sti   R3,@S0_xdata    ; Output the new DAC value
          pop   R3              
          pop   ST              
          reti                  
;----------------------------------------------------------------
ADC2      push  ST              
          push  R3              
          push  AR0
          push R1
          ldi   @S0_rdata,R3    
          lsh   16,R3
          ash   -18,R3
; La tabla se carga con 100  valores
; y luego se saltea este codigo
          ldi  @veces,R1
          cmpi 0,R1
          bz    seguir2
          subi 1,R1
          sti  R1,@veces
          ldi  @ultimo,AR0
          sti  R3,*AR0++
          sti  AR0,@ultimo
seguir    sti   R3,@ADC_last
          pop   R1
          pop   AR0
          pop   R3              
          pop   ST              
          reti                  
seguir2   b seguir              
;******************************************************************;
; Esta parte del programa se usa durante la inicializacion solamente
; y se puede sobreescribir sin problemas por el stack o datos.
;******************************************************************;
          .entry   ST_STUB      ; Debugger starts here
;
ST_STUB   ldp   T0_ctrl
; Carga el puntero de pagina  de datos DP (data Page)
; con los 8 bits  mas significativos
; del T0_ctrl ( =808020h) o sea con el 80 ,
; definiendo de esta manera que se use la data
; page y el stack del Kernel .
;
          ldi   @stack,SP
; Carga el SP con la direccion del lugar asignado con la directiva
; que se usa mas adelante que indica que el stack comienza ahi.
; Se programa el timer cero que se usa para generar el Master clock.
; Este valor  define despues (ademas de los registros A y B del AIC)
; las frecuencias de conversion de  los ADC  y DACs.;
; Notar que el timer cero se ha programado
; en modo pulso con el periodo en dos.
; Por lo tanto, la frecuencia de salida en modo pulso es
; (fsp) =  f_timer clock (entrada) / registro de periodo =
; = (50mhz/4)/2= 12.5 mhz /2 = 6.25MHz
;
          ldi   0,R0            ; Halt TIM0 & TIM1
          sti   R0,@T0_ctrl     ;
          sti   R0,@T0_count    ; Set counts to 0
          ldi   2,R0            ; Set periods to 2
          sti   R0,@T0_prd      
;
; GO=1 hace el reset y comienzo de la cuenta y HLDneg =1
; quita el "hold" (en la palabra de control del timer cero = T0_ctrl)
;
          ldi   0x2C1,R0        ; Restart both timers
          sti   R0,@T0_ctrl     ;
;
;----------------------------------------------------------
;   Programacion del registro del control de la transmision 
;
          ldi   @S0_xctrl_val,R0;
          sti   R0,@S0_xctrl    ;
; Transmit control: Con FSX/DX/CLKX en modo de transmision de puerta serie
; (podrian usarse como simples pines de E/S).
;
; Programacion del registro de control de RECEPCION,
; definiendo los pines FSR/DR/CLKR para comunicacion serie (no E/S)
          ldi   @S0_rctrl_val,R0;
          sti   R0,@S0_rctrl    ; receive control
; Transmite un cero
          ldi   0,R0            ;
          sti   R0,@S0_xdata    ; DXR data value
;
; Programa el global control register para la puerta serie.
; El 'C31 le envia al conversor (AIC) los datos en serie.
; Notar que se le carga una palabra definida al principio del programa
; (0x0E970300) y se define que la fuente de reloj para la Tx y Rx
; sea externa, en este caso la genera el AIC.
;
          ldi   @S0_gctrl_val,R0; Setup serial port
          sti   R0,@S0_gctrl    ; global control
;======================================================;
; Esta seccion inicializa el  AIC             
;======================================================;
AIC_INIT  LDI   0x10,IE         ; Enable only XINT interrupt
          andn  0x34,IF         ; deshabilita las int. int2, xint0 rint0
          ldi   0,R0            ; transmite un cero
          sti   R0,@S0_xdata   
; Por el pin XF0  (controlado por el registro IOF)
; saca un cero y luego un uno, escalon necesario para
; resetear el AIC y luego  ponerlo en RUN
;
          RPTS  0x040           
          LDI   2,IOF           ; XF0=0 resets AIC
          rpts  0x40            
          LDI   6,IOF           ; XF0=1 runs AIC
          ;---------------------
          ldi   @C_REG,R0       ; Setup control register
          call  prog_AIC        
          ldi   0xfffc  ,R0     ; Program the AIC to be real slow
          call  prog_AIC        
          ldi   0xfffc|2,R0     
          call  prog_AIC        
          ldi   @B_REG,R0       ; Bump up the Fs to final rate
          call  prog_AIC        ; (smallest divisor should be last)
          ldi   @A_REG,R0       
          call  prog_AIC
          b     main
;-------------------------------
prog_AIC  ldi   @S0_xdata,R1    ; Use original DXR data during 2 ndy
          sti   R1,@S0_xdata
          idle
          ldi   @S0_xdata,R1    ; Use original DXR data during 2 ndy
          or    3,R1            ; Request 2 ndy XMIT
          sti   R1,@S0_xdata    
          idle                  
          sti   R0,@S0_xdata    ; Send register value
          idle                  
          andn  3,R1            
          sti   R1,@S0_xdata    ; Leave with original safe value in DXR
          ;---------------------
          ldi   @S0_rdata,R0    ; Fix the receiver underrun by reading
          rets                  ; the DRR before going to the main loop
TABLA   .space  120
stack     .word   $             ; Put stack here
;****************************************************;
; Install the XINT/RINT ISR handler directly into    ;
; the vector RAM location it will be used for        ;
;****************************************************;
          .start   "SP0VECTS",0x809FC5
          .sect    "SP0VECTS"
          B        DAC2         ; XINT0
          B        ADC2         ; RINT0
                                     
